<?php
/* This file is part of a copyrighted work; it is distributed with NO WARRANTY.
 * See the file COPYRIGHT.html for more details.
 */
 
require_once("../shared/global_constants.php");
require_once("../classes/Query.php");

class InstallQuery extends Query {
  /* Override constructor so the installer can test the database connection */
  function InstallQuery() {
    ;
  }
  function dropTable($tableName) {
    $sql = $this->mkSQL("drop table if exists %I ", $tableName);
    $this->exec($sql);
  }
  
  function renameTables($fromTablePrfx, $toTablePrfx = DB_TABLENAME_PREFIX) {
    $fromTableNames = $this->getTableNames($fromTablePrfx.'%');
    foreach($fromTableNames as $fromTableName) {
      $toTableName = str_replace($fromTablePrfx, $toTablePrfx, $fromTableName);
      $this->renameTable($fromTableName, $toTableName);
    }
  }
  
  function renameTable($fromTableName, $toTableName) {
      $this->dropTable($toTableName);
      $sql = "rename table ".$fromTableName." to ".$toTableName;
      $this->exec($sql);
  }

  function getTableNames($pattern = "") {
    if($pattern == "") {
      $pattern = DB_TABLENAME_PREFIX.'%';
    }
    $sql = "show tables like '".$pattern."'";
    $rows = $this->exec($sql, OBIB_NUM);

    $tablenames = array();
    foreach ($rows as $row) {
      $tablenames[] = $row[0];
    }
    return $tablenames;
  }
  
  function _getSettings($tablePrfx) {
    $sql = $this->mkSQL('SHOW TABLES LIKE %Q ', $tablePrfx.'settings');
    $row = $this->select01($sql);
    if (!$row) {
      return false;
    }
    $sql = $this->mkSQL('SELECT * FROM %I ', $tablePrfx.'settings');
    $row = $this->select01($sql);
    if (!$row) {
        // if settings empty - looks like it not initialised
      return false;
    }
    return $row;
  }
  
  function getCurrentLocale($tablePrfx = DB_TABLENAME_PREFIX) {
    $array = $this->_getSettings($tablePrfx);
    if($array == false ||
      !isset($array["locale"])) {
      return 'en'; //Earlier versions of Openbiblio only supported English
    }
    else {
      return $array["locale"];  
    }
  }

  function getCurrentCharset($tablePrfx = DB_TABLENAME_PREFIX) {
    $array = $this->_getSettings($tablePrfx);
    if($array == false ||
      !isset($array["charset"])) {
      return 'utf-8'; //Earlier versions of Openbiblio only supported English
    }
    else {
      return $array["charset"];  
    }
  }
  
  function getCurrentDatabaseVersion($tablePrfx = DB_TABLENAME_PREFIX) {
    $array = $this->_getSettings($tablePrfx);
    if($array == false) {
      return false;
    }
    else {
      return $array["version"];
    }
  }
  
  function freshInstall($locale, $sampleDataRequired = false,
                        $version=OBIB_LATEST_DB_VERSION,
                        $tablePrfx = DB_TABLENAME_PREFIX) {
    $rootDir = '../install/' . $version . '/sql';
    $localeDir = '../locale/' . $locale . '/sql/' . $version;
    include($localeDir."/domain/sqlcharset.php");
    $this->UseLinkCharset($sql_charset);
    
    $this->executeSqlFilesInDir($rootDir, $tablePrfx);
    $this->executeSqlFilesInDir($localeDir . '/domain', $tablePrfx);
    if($sampleDataRequired) {
      $this->executeSqlFilesInDir($localeDir . '/sample', $tablePrfx);
    }
    $sql = $this->mkSQL('update ignore '.$tablePrfx.'lang set default_flg=\'Y\' where id = %Q ', $locale);
    $this->act($sql);
  }
  
  function getSqlFilesInDir($dir, $skipfiles, $tablePrfx = "") {
    if (is_dir($dir)) {
      if ($dh = opendir($dir)) {
        $files = array();
        while (($filename = readdir($dh)) !== false) {
          if(preg_match('/\\.sql$/', $filename)) {
            if ($skipfiles == false)
                $files[] = $filename;
            else if (!in_array($filename, $skipfiles))
                $files[] = $filename;
          }
        }
        closedir($dh);
        if (count($files) == 0)
            return false;
        return $files;
      }
    }
    return false;
  }

  function executeSqlFilesInDir($dir, $tablePrfx = "") {
    $files = $this->getSqlFilesInDir($dir, false ,$tablePrfx);
    if ($files == false)
        return false;
    sort($files, SORT_STRING);
    $this->executeSqlFiles($dir, $files, $tablePrfx);
    return true;
  }

  function executeSqlFilesInDirSkips($dir, $skipfiles, $tablePrfx = "") {
    $files = $this->getSqlFilesInDir($dir, $skipfiles, $tablePrfx);
    if ($files == false)
        return false;
    sort($files, SORT_STRING);
    $this->executeSqlFiles($dir, $files, $tablePrfx);
    return true;
  }
  /**********************************************************************************
   * Function to read through an sql file executing SQL only when ";" is encountered
   **********************************************************************************/
  function executeSqlFiles($dir, $files, $tablePrfx = DB_TABLENAME_PREFIX, $Quiet = false) {
        for($i=0; $i<count($files); $i++) {
            $filename = $files[$i];
            $this->executeSqlFile($dir.'/'.$filename, $tablePrfx, $Quiet);
        }
  }

  function PreprocessSQL($sql){
    //preprocess macro and params in sql
    return $sql;
  }

  function executeSqlFile($filename, $tablePrfx = DB_TABLENAME_PREFIX, $Quiet = false) {
    $fp = false;
    if (file_exists($filename))
        $fp = fopen($filename, "r");
    # show error if file could not be opened
    if ($fp == false) {
      if ($Quiet) {
            return false;
      }
      Fatal::error("Error reading file: ".H($filename));
    } else {
      $sqlStmt = "";
      while (!feof ($fp)) {
        $char = fgetc($fp);
        if ($char == ";") {
          //replace table prefix, we're doing it here as the install script may
          //want to override the required prefix (eg. during upgrade / conversion 
          //process)
          $sql = str_replace("%prfx%",$tablePrfx,$sqlStmt);
          $sql = $this->PreprocessSQL($sql);
          //replace ENGINE with TYPE for old MySQL versions
          $MySQLn = explode('.', implode('', explode('-', mysql_get_server_info())));
          if ($MySQLn[0] < '5') {
            $sql = str_replace("ENGINE=","TYPE=",$sql);
            $sql = str_replace("engine=","type=",$sql);
          } else {
            $sql = str_replace("TYPE=","ENGINE=",$sql);
            $sql = str_replace("type=","engine=",$sql);
          }
          $this->exec($sql);
          $sqlStmt = "";
        } else {
          $sqlStmt .= $char;
        }
      }
      fclose($fp);
    }
    return true;
  }
}

?>
