<?php
/* This file is part of a copyrighted work; it is distributed with NO WARRANTY.
 * See the file COPYRIGHT.html for more details.
 */
 
require_once("../shared/global_constants.php");
require_once("../classes/InstallQuery.php");

class LocaleQuery extends InstallQuery {
  public    $_locale = "en";
  public    $_isdefaultlocale = false;
  public    $_charset = '';
  public    $_version = '0.8.0';
  public    $tablePrfx = DB_TABLENAME_PREFIX;
  public    $localekey = -1;

  /* Override constructor so the installer can test the database connection */
  function __construct($locale, $charset='', $prfx = DB_TABLENAME_PREFIX, $version = '') {
        parent::__construct();
        $this->Query();
        
        $this->_locale    = $locale;
        $this->_charset   = $charset;
        $this->_version   = $version;
        $this->tablePrfx  = $prfx;
  }

  function _readLocaleInfo() {
//    $sql = $this->mkSQL('SHOW TABLES LIKE %Q ', $tablePrfx.'lang');
//    $row = $this->select01($sql);
//    if (!$row) {
//      return false;
//    }
    $sql = $this->mkSQL('SELECT * FROM %I WHERE id = %Q', $this->tablePrfx.'lang', $this->_locale);
    return $this->select01($sql);
  }
  
  function _readDefaultLocaleInfo() {
//    $sql = $this->mkSQL('SHOW TABLES LIKE %Q ', $tablePrfx.'lang');
//    $row = $this->select01($sql);
//    if (!$row) {
//      return false;
//    }
    $sql = $this->mkSQL('SELECT * FROM %I WHERE default_flg = "Y"', $this->tablePrfx.'lang');
    return $this->select01($sql);
  }

  //read ifo of locale from lang_dm table
  function GetDefaultLocale() {
    $array = $this->_readDefaultLocaleInfo();
    if($array == false ||
      !isset($array["id"])) {
      return 'en'; //Earlier versions of Openbiblio only supported English
    }
    else {
        $this->_locale = $array["id"];
        $this->localekey = $array["code"];
        //$this->_charset  = $array["charset"];
        $this->_isdefaultlocale = ($array["default_flg"] == "Y");
      return $array["id"];  
    }
  }
  
  //read ifo of locale from lang_dm table
  function _getLocaleInfo() {
    $array = $this->_readLocaleInfo();
    if($array == false ||
      !isset($array["id"])) {
      return 'en'; //Earlier versions of Openbiblio only supported English
    }
    else {
        $this->localekey = $array["code"];
        $this->_isdefaultlocale = ($array["default_flg"] == "Y");
      return $array["id"];  
    }
  }

  function PrepareLocale() {
    $array = $this->_readLocaleInfo();
    if ($array != false) {
        $this->localekey = $array["code"];
        $this->_isdefaultlocale = ($array["default_flg"] == "Y");
        return $this->UpdateLocale();
    }
    if (!$this->freshInstallLocale()){
        return false;
    }
    if ($this->localekey < 0){
        return false;
    }
    return true;
  }
  
  function PrepareLocaleSeance() {
    $result = $this->PrepareLocale( );
    return $result;
  }
 
  function UpdateLocale(){
    return $this->freshInstallLocale();
  }

  function freshInstallLocale() {
    if ($this->_isdefaultlocale) 
        return true;

    if ($this->_version == ''){
        $this->_version = $this->getCurrentDatabaseVersion($this->tablePrfx);
    }
    $localeDir = '../locale/' . $this->_locale . '/sql/' . $this->_version;
    if (is_dir($localeDir) ) {
    include($localeDir."/append/sqlcharset.php");
    $this->UseLinkCharset($sql_charset);

    $lang_sqls[] = 'lang.sql';
    $this->executeSqlFiles($localeDir.'/append', $lang_sqls, $this->tablePrfx, true);
    $array = $this->_readLocaleInfo();
    if ($array == false){
        Fatal::error("fail _readLocaleInfo: ");
        return false;
    }
    $this->localekey = $array["code"];
    $this->executeSqlFilesInDirSkips($localeDir . '/append', $lang_sqls, $this->tablePrfx);
    }
    else{
        Fatal::error("DB sql for desired local ".$this->_locale ." absent");
    }
    $this->GenerateLocaleInterface();
    return true;
  }

  function PreprocessSQL($sql){
    if ($this->localekey >= 0)
        $sql = str_replace("%localeid%",$this->localekey,$sql);
    return $sql;
  }

  function getLocaleCharset(){
    return $this->_charset;
  }
  
  function GenLocalTable_PrepareDescr($table, $local_descr, $descrlen = 124){
        $use_charset = "";
        if (strlen(DB_CHARSET)> 0)
            $use_charset = 'CHARACTER SET '.$this->CharSet2MySQL(DB_CHARSET);
        $sql = $this->mkSQL('alter ignore table %I add %C varchar(%N) %! NULL default NULL', $table, $local_descr, $descrlen, $use_charset);
        $result = $this->tryact($sql);
       
        if ($result === false){
            $sql = $this->mkSQL('alter ignore table %I change %C %C varchar(%N) %! NULL default NULL', $table, $local_descr, $local_descr, $descrlen, $use_charset);
            $result = $this->tryact($sql);
        }
        //$this->act($sql);
        return $result;
  }
  
  function SelectLocalsSql($table, $descr = 'description'){
        $use_tabl_def = $table;
        if ($descr != 'description')
            $use_tabl_def .= '_'.$descr;
        $sql_locals = $this->mkSQL('select code, tabl_code, tabl_scode, description from locale_describe as lang where ((lang.langid = %N) and (lang.tabl_name = %Q))', $this->localekey, $use_tabl_def);
        return $sql_locals;
  }
  
  function GenLocalTable_StrKey($table , $key, $descr = 'description'){
        $local_descr = $descr.'_'.$this->_locale;
        $this->GenLocalTable_PrepareDescr($table, $local_descr);
        $sql_locals = $this->SelectLocalsSql($table, $descr);
        $sql_upd_locals = $this->mkSQL('update %I as dst, (%!) as src set dst.%i = src.description where (dst.%i = src.tabl_scode)'
                                        , $table, $sql_locals, $local_descr, $key);//, $local_descr);// and isnull(dst.%i))
        $result = $this->act($sql_upd_locals);
        return $result;
  }

    function GenLocalTable_IntKey($table , $key, $descr = 'description'){
        $local_descr = $descr.'_'.$this->_locale;
        $this->GenLocalTable_PrepareDescr($table, $local_descr);
        $sql_locals = $this->SelectLocalsSql($table, $descr);
        $sql_upd_locals = $this->mkSQL('update %I as dst, (%!) as src set dst.%i = src.description where (dst.%i = src.tabl_code)'
                                        , $table, $sql_locals, $local_descr, $key);//, $local_descr);// and isnull(dst.%i)
        $result = $this->act($sql_upd_locals);
        return $result;
    }

    function LocaliseKeyExpr($key, $table_prefix = ''){
        //ru: генерит выражение строковое - контеканацию всех полей в $key, 
        //при этом числовые поля переводятся к 10ного вида строке - они маркируются знаком  # <например #tag>
        //маркировка этих полей не обязательна
        $result = '';
        for($i=0; $i<count($key); $i++) {
            $field = $key[$i];
            if ($result != '')
                $result .= ',\' \',';
            if ($field[0] == '#'){
                $field = ltrim($field, '#');
            }
            $result .= $table_prefix.$field;
        }
        $result = 'concat('.$result.')';
        return $result;
    }
    
    function GenLocalTable_CombKey($table , $key, $descr = 'description'){
        $local_descr = $descr.'_'.$this->_locale;
        $this->GenLocalTable_PrepareDescr($table, $local_descr);
        $sql_locals = $this->SelectLocalsSql($table, $descr);
        $keyexpr = $this-> LocaliseKeyExpr($key, 'dst.');
        $sql_upd_locals = $this->mkSQL('update %I as dst, (%!) as src set dst.%i = src.description where (%! = src.tabl_scode)'
                                        , $table, $sql_locals, $local_descr, $keyexpr);//, $local_descr);// and isnull(dst.%i)
        $result = $this->act($sql_upd_locals);
        return $result;
    }
    
  function GenerateLocaleInterface(){
    if ($this->_isdefaultlocale) return true;
    //generate tables\views wis localised 'description' fields in form <localeid>_<tablename>
    $this->GenLocalTable_StrKey('biblio_copy_fields_dm' , 'code');
    $this->GenLocalTable_StrKey('biblio_status_dm'      , 'code');
    $this->GenLocalTable_IntKey('material_type_dm'      , 'code');
    $this->GenLocalTable_StrKey('collection_dm'         , 'description');
    $this->GenLocalTable_IntKey('material_usmarc_xref'  , 'xref_id', 'descr');
    
    $this->GenLocalTable_StrKey('mbr_classify_dm'       , 'description');
    $this->GenLocalTable_StrKey('member_fields_dm'      , 'code');
    $this->GenLocalTable_StrKey('transaction_type_dm'   , 'code');
    $this->GenLocalTable_IntKey('usmarc_block_dm'       , 'block_nmbr');

    $this->GenLocalTable_CombKey('usmarc_subfield_dm'   , ['#tag', 'subfield_cd']);
    $this->GenLocalTable_CombKey('usmarc_indicator_dm'  , ['#tag', 'indicator_nmbr', 'indicator_cd']);
    $this->GenLocalTable_CombKey('usmarc_tag_dm'        , ['#tag', '#block_nmbr']);
    $this->GenLocalTable_CombKey('usmarc_tag_dm'        , ['#tag', '#block_nmbr'], 'ind1_description');
    $this->GenLocalTable_CombKey('usmarc_tag_dm'        , ['#tag', '#block_nmbr'], 'ind2_description');
    
    return true;
  }
}

?>
